@@ -53,7 +53,6 @@ extern char *strdup(const char *);
 #include <stdio.h>
 #include <utmp.h>
 #include <grp.h>
-#include <glib/galloca.h>
 #include "gnome-pty.h"
 #include "gnome-login-support.h"
 
@@ -95,20 +94,16 @@ static pty_info *pty_list;
 #endif
 #endif /* CMSG_DATA */
 
-static struct cmsghdr *cmptr;
-static int CONTROLLEN;
-
-static int
-init_msg_pass (void)
-{
-	CONTROLLEN = (CMSG_DATA (cmptr) - (unsigned char *)cmptr) + sizeof(int);
-	cmptr = malloc (CONTROLLEN);
-
-	if (cmptr)
-		return 0;
-
-	return -1;
-}
+/* Solaris doesn't define these */
+#ifndef CMSG_ALIGN
+#define CMSG_ALIGN(len) (((len) + sizeof (size_t) - 1) & (size_t) ~(sizeof (size_t) - 1))
+#endif
+#ifndef CMSG_SPACE
+#define CMSG_SPACE(len) (CMSG_ALIGN (len) + CMSG_ALIGN (sizeof (struct cmsghdr)))
+#endif
+#ifndef CMSG_LEN
+#define CMSG_LEN(len) (CMSG_ALIGN (sizeof (struct cmsghdr)) + (len))
+#endif
 
 static int
 pass_fd (int client_fd, int fd)
@@ -116,6 +111,8 @@ pass_fd (int client_fd, int fd)
         struct iovec  iov[1];
         struct msghdr msg;
         char          buf [1];
+        char    cmsgbuf[CMSG_SPACE(sizeof(int))];
+        struct  cmsghdr *cmptr;
 
 	iov [0].iov_base = buf;
 	iov [0].iov_len  = 1;
@@ -124,12 +121,13 @@ pass_fd (int client_fd, int fd)
 	msg.msg_iovlen     = 1;
 	msg.msg_name       = NULL;
 	msg.msg_namelen    = 0;
-	msg.msg_control    = (caddr_t) cmptr;
-	msg.msg_controllen = CONTROLLEN;
+	msg.msg_control    = (caddr_t) cmsgbuf;
+	msg.msg_controllen = sizeof(cmsgbuf);
 
+        cmptr = CMSG_FIRSTHDR(&msg);
 	cmptr->cmsg_level = SOL_SOCKET;
 	cmptr->cmsg_type  = SCM_RIGHTS;
-	cmptr->cmsg_len   = CONTROLLEN;
+	cmptr->cmsg_len   = CMSG_LEN(sizeof(int));
 	*(int *)CMSG_DATA (cmptr) = fd;
 
 	if (sendmsg (client_fd, &msg, 0) != 1)
@@ -153,12 +151,6 @@ pass_fd (int client_fd, int fd)
 #include <sys/socket.h>
 #include <sys/uio.h>
 
-static int
-init_msg_pass ()
-{
-  return 0;
-}
-
 static int
 pass_fd (int client_fd, int fd)
 {
@@ -185,12 +177,6 @@ pass_fd (int client_fd, int fd)
 #else
 #include <stropts.h>
 #ifdef I_SENDFD
-static int
-init_msg_pass ()
-{
-	/* nothing */
-	return 0;
-}
 
 int
 pass_fd (int client_fd, int fd)
@@ -284,20 +270,6 @@ pty_add (int utmp, int wtmp, int lastlog, char *line, char *login_name)
 	return pi;
 }
 
-static int
-path_max (void)
-{
-#ifdef _PC_PATH_MAX
-	return pathconf ("/", _PC_PATH_MAX);
-#else
-#  ifdef PATH_MAX
-	return PATH_MAX;
-#  else
-	return 1024;
-#  endif
-#endif
-}
-
 static struct termios*
 init_term_with_defaults(struct termios* term)
 {
@@ -467,7 +439,7 @@ init_term_with_defaults(struct termios* term)
 static int
 open_ptys (int utmp, int wtmp, int lastlog)
 {
-	char *term_name;
+	const char *term_name;
 	int status, master_pty, slave_pty;
 	pty_info *p;
 	int result;
@@ -476,11 +448,6 @@ open_ptys (int utmp, int wtmp, int lastlog)
 	struct group *group_info;
 	struct termios term;
 
-	term_name = (char *) g_alloca (path_max () + 1);
-
-	if (term_name == NULL) {
-		exit (1);
-	}
 	/* Initialize term */
 	init_term_with_defaults(&term);
 
@@ -499,7 +466,7 @@ open_ptys (int utmp, int wtmp, int lastlog)
 #error "No means to drop privileges! Huge security risk! Won't compile."
 #endif
 	/* Open pty with privileges of the user */
-	status = openpty (&master_pty, &slave_pty, term_name, &term, NULL);
+	status = openpty (&master_pty, &slave_pty, NULL, &term, NULL);
 
 	/* Restore saved privileges to root */
 #ifdef HAVE_SETEUID
@@ -512,7 +479,7 @@ open_ptys (int utmp, int wtmp, int lastlog)
 #error "No means to raise privileges! Huge security risk! Won't compile."
 #endif
 	/* openpty() failed, reject request */
-	if (status == -1) {
+	if (status == -1 || (term_name = ttyname(slave_pty)) == NULL) {
 		result = 0;
 		n_write (STDIN_FILENO, &result, sizeof (result));
 		return 0;
@@ -658,12 +625,12 @@ sanity_checks (void)
 	}
 }
 
-static gboolean done;
+static volatile int done;
 
 static void
 exit_handler (int signum)
 {
-	done = TRUE;
+	done = 1;
 }
 
 
@@ -702,20 +669,17 @@ main (int argc, char *argv [])
 	if (!display_name)
 		display_name = "localhost";
 
-	done = FALSE;
+	done = 0;
 
 	/* Make sure we clean up utmp/wtmp even under vncserver */
 	signal (SIGHUP, exit_handler);
 	signal (SIGTERM, exit_handler);
 
-	if (init_msg_pass () == -1)
-		exit (1);
-
 	while (!done) {
 		res = n_read (STDIN_FILENO, &op, sizeof (op));
 
 		if (res != sizeof (op)) {
-			done = TRUE;
+			done = 1;
 			continue;
 		}
 
@@ -763,6 +727,13 @@ main (int argc, char *argv [])
 			}
 			close_pty_pair (tag);
 			break;
+
+		case GNOME_PTY_SYNCH:
+			{
+				int result = 0;
+				n_write (STDIN_FILENO, &result, 1);
+			}
+			break;
 		}
 
 	}
